@@ -4,32 +4,60 @@
 LDAPCSDK_CO_TAG = 'LDAPCSDK_6_0_6D_MOZILLA_RTM'
 LDAPCSDK_DIRS = ('directory/c-sdk',)
 
-DEFAULT_COMM_REV = "default"
-
-# URL of the default hg repository to clone for Mozilla.
-DEFAULT_MOZILLA_REPO = 'http://hg.mozilla.org/releases/mozilla-1.9.1/'
-DEFAULT_MOZILLA_REV = "default"
-
-# REGEX to match against, $1 should refer to protocol scheme
-MOZILLA_TRUNK_REPO_REGEXP = "(ssh|http|https):\/\/hg\.mozilla\.org\/mozilla-central\/?$"
-MOZILLA_BASE_REV = "GECKO_1_9_1_BASE"
-
 # URL of the default hg repository to clone for ChatZilla.
 DEFAULT_CHATZILLA_REPO = 'http://hg.mozilla.org/chatzilla/'
-DEFAULT_CHATZILLA_REV = "COMM_1_9_1_BRANCH"
+DEFAULT_CHATZILLA_REV = "default"
 
 # URL of the default hg repository to clone for DOM Inspector.
 DEFAULT_INSPECTOR_REPO = 'http://hg.mozilla.org/dom-inspector/'
-DEFAULT_INSPECTOR_REV = "COMM_1_9_1_BRANCH"
+DEFAULT_INSPECTOR_REV = "COMM_1_9_2_BRANCH"
 
 # URL of the default hg repository to clone for Venkman.
 DEFAULT_VENKMAN_REPO = 'http://hg.mozilla.org/venkman/'
-DEFAULT_VENKMAN_REV = "COMM_1_9_1_BRANCH"
+DEFAULT_VENKMAN_REV = "default"
+
+DEFAULT_COMM_REV = "default"
+
+# URL of the default hg repository to clone for Mozilla.
+DEFAULT_MOZILLA_REPO = 'http://hg.mozilla.org/releases/mozilla-1.9.2/'
+DEFAULT_MOZILLA_REV = "default"
+
+# The set of defaults below relate to the current switching mechanism between
+# trunk or branches and back again if it is required.
+
+# The current version expected in the .treestate file for nothing to happen.
+# This reflects the "[treestate] src_update_version = ..." value.
+#
+# src_update_version values:
+# '1' : mozilla/ may have been moved to mozilla-1.9.1 (or kept the same
+#       depending on set-up).
+# '2' : mozilla/ may have been moved back to mozilla-central.
+CURRENT_TREESTATE_VERSION = '2'
+# REGEX to match against, $1 should refer to protocol scheme
+SWITCH_MOZILLA_REPO_REGEXP = '(ssh|http|https):\/\/hg\.mozilla\.org\/releases/mozilla-1.9.1\/?$'
+# The location to back-up the existing mozilla repo to, e.g. ".mozilla-trunk"
+# or ".mozilla-1.9.1".
+SWITCH_MOZILLA_REPO_BACKUP_LOCATION = ".mozilla-1.9.1"
+# This is the potential location for a repository from the last time we
+# switched. Can be blank for no effect.
+SWITCH_MOZILLA_REPO_OLD_REPO_LOCATION = ".mozilla-trunk"
+# This should be the same as DEFAULT_MOZILLA_REPO but using %s instead of http
+# for the scheme.
+SWITCH_MOZILLA_REPO_REPLACE = '%s://hg.mozilla.org/mozilla-central/'
+SWITCH_MOZILLA_BASE_REV = "GECKO_1_9_1_BASE"
+
+import sys
+# Test Python Version. 2.4 required for `import subprocess`
+pyver = sys.version_info
+if pyver[0] <= 1 or (pyver[0] == 2 and pyver[1] < 4):
+  sys.exit("ERROR: Python 2.4 or newer required")
+elif pyver[0] >= 3:
+  sys.exit("ERROR: Python series 3 is not supported, use series 2 > 2.4")
+del pyver
 
 import os
-import sys
 import datetime
-from optparse import OptionParser
+from optparse import OptionParser, OptionValueError
 
 topsrcdir = os.path.dirname(__file__)
 if topsrcdir == '':
@@ -49,52 +77,98 @@ except ImportError:
                 cmd = popenargs[0]
                 raise Exception("Command '%s' returned non-zero exit status %i" % (cmd, retcode))
 
-def check_call_noisy(cmd, *args, **kwargs):
+def check_call_noisy(cmd, retryMax=0, *args, **kwargs):
+  """Wrapper around execute_check_call() to allow retries before failing.
+
+  |cmd|, is the command to try and execute.
+  |retryMax|, is the maximum number of retries to attempt, 0 by default.
+  """
+
+  def execute_check_call(cmd, *args, **kwargs):
     print "Executing command:", cmd
     check_call(cmd, *args, **kwargs)
 
+  # By default (= no retries), simply pass the call on.
+  if retryMax == 0:
+    execute_check_call(cmd, *args, **kwargs)
+    return
+
+  # Loop (1 + retryMax) times, at most.
+  for r in range(0, 1 + retryMax):
+    # Add a retry header, not for initial call.
+    if r != 0:
+      print >> sys.stderr, "Retrying previous command: %d of %d time(s)" % (r, retryMax)
+    try:
+      # (Re-)Try the call.
+      execute_check_call(cmd, *args, **kwargs)
+      # If the call succeeded then no more retries.
+      break
+    except:
+      # Print the exception without its traceback.
+      # This traceback starts in the try block, which should be low value.
+      print >> sys.stderr, "The exception was:"
+      sys.excepthook(sys.exc_info()[0], sys.exc_info()[1], None)
+      # Add a blank line.
+      print >> sys.stderr
+  else:
+    # Raise our own exception.
+    # This traceback starts at (= reports) the initial caller line.
+    raise Exception("Command '%s' failed %d time(s). Giving up." % (cmd, retryMax + 1))
+
 def repo_config():
     """Create/Update TREE_STATE_FILE as needed.
 
-    move_to_stable() is also called.
+    switch_mozilla_repo() may be called if the branch that the mozilla repo is
+    on needs changing
     """
 
-    move_to_stable()
-
     import ConfigParser
-    config = ConfigParser.ConfigParser()
-    config.read([TREE_STATE_FILE])
 
-    # 'src_update_version' values:
-    #   '1': "move_to_stable() was successfully run".
+    config_version = CURRENT_TREESTATE_VERSION
+
+    config = ConfigParser.ConfigParser()
+
+    # If the file exists, see if we need to move to the stable branch.
+    # If it doesn't exist, we assume the user hasn't pulled before (or has
+    # done it manually) and therefore leave his mozilla/ repo as it is.
+    if os.path.exists(TREE_STATE_FILE):
+        config.read([TREE_STATE_FILE])
+
+        if config.has_option('treestate', 'src_update_version'):
+            config_version = config.get('treestate', 'src_update_version')
+        else:
+            # If src_update_version doesn't exist, assume this to be '1' and
+            # apply the update. This was because the first version of this
+            # migration allowed the existence of the file to equate to '1'.
+            config_version = '1'
 
     # Do nothing if the current version is up to date.
-    if config.has_option('treestate', 'src_update_version') and \
-            config.get('treestate', 'src_update_version') == '1':
-        return
+    if config_version < CURRENT_TREESTATE_VERSION:
+        switch_mozilla_repo()
 
-    if not config.has_section('treestate'):
-      config.add_section('treestate')
-    config.set('treestate', 'src_update_version', '1')
+    if not os.path.exists(TREE_STATE_FILE) or config_version < CURRENT_TREESTATE_VERSION:
+        if not config.has_section('treestate'):
+            config.add_section('treestate')
+        config.set('treestate', 'src_update_version', CURRENT_TREESTATE_VERSION)
 
-    # Write this file out
-    f = open(TREE_STATE_FILE, 'w')
-    try:
-      config.write(f)
-    finally:
-      f.close()
+        # Write this file out
+        f = open(TREE_STATE_FILE, 'w')
+        try:
+            config.write(f)
+        finally:
+            f.close()
 
-def move_to_stable():
-    """Backup (unused anymore) trunk checkout of Mozilla.
+def switch_mozilla_repo():
+    """If the mozilla/ repo matches SWITCH_MOZILLA_REPO_REGEXP then:
+    1) Backup (unused anymore) checkout of mozilla/.
+    2a) If SWITCH_MOZILLA_REPO_OLD_REPO_LOCATION exists, move that to mozilla/.
+    2b) Else clone the backup of mozilla/ up to the SWITCH_MOZILLA_BASE_REV
+        revision and set the pull location to SWITCH_MOZILLA_REPO_REPLACE.
 
-    Also switch checkout to MOZILLA_BASE_REV of Mozilla 1.9.1.
+    It is expected that the normal pull/update functions in this script will
+    update the new mozilla/ repo to the latest version.
     """
 
-    # Do nothing if this function was already successfully run.
-    # Shortcut: checking file existence is enough.
-    if os.path.exists(TREE_STATE_FILE):
-        return
-
     mozilla_path = os.path.join(topsrcdir, 'mozilla')
     # Do nothing if there is no Mozilla directory.
     if not os.path.exists(mozilla_path):
@@ -107,38 +181,59 @@ def move_to_stable():
         # Abort, not to get into a possibly inconsistent state.
         sys.exit("Error: default path in mozilla/.hg/hgrc is undefined!")
 
-    # Compile the Mozilla trunk regex.
-    m_c_regex = re.compile(MOZILLA_TRUNK_REPO_REGEXP, re.I)
-    match = m_c_regex.match(config.get('paths', 'default'))
-    # Do nothing if not pulling from Mozilla trunk.
+    # Compile the Mozilla repository regex.
+    moz_old_regex = re.compile(SWITCH_MOZILLA_REPO_REGEXP, re.I)
+    match = moz_old_regex.match(config.get('paths', 'default'))
+    # Do nothing if not pulling from the one we're trying to switch from.
     if not match:
         return
 
     config.set('paths', 'default',
-               "%s://hg.mozilla.org/releases/mozilla-1.9.1/" % match.group(1) )
+               SWITCH_MOZILLA_REPO_REPLACE % match.group(1) )
 
     if config.has_option('paths', 'default-push'):
-      match = m_c_regex.match(config.get('paths', 'default-push'))
+      match = moz_old_regex.match(config.get('paths', 'default-push'))
       # Do not update this property if not pushing to Mozilla trunk.
       if match:
         config.set('paths', 'default-push',
-                   "%s://hg.mozilla.org/releases/mozilla-1.9.1/" % match.group(1) )
+                   SWITCH_MOZILLA_REPO_REPLACE % match.group(1) )
+
+    hgcloneopts = []
+    if options.hgcloneopts:
+        hgcloneopts = options.hgcloneopts.split()
 
     hgopts = []
     if options.hgopts:
         hgopts = options.hgopts.split()
 
-    mozilla_trunk_path = os.path.join(topsrcdir, '.mozilla-trunk')
-    print "Moving mozilla to .mozilla-trunk..."
+    backup_mozilla_path = os.path.join(topsrcdir, SWITCH_MOZILLA_REPO_BACKUP_LOCATION)
+    print "Moving mozilla to " + SWITCH_MOZILLA_REPO_BACKUP_LOCATION + "..."
     try:
-        os.rename(mozilla_path, mozilla_trunk_path)
+        os.rename(mozilla_path, backup_mozilla_path)
     except:
         # Print the exception without its traceback.
         sys.excepthook(sys.exc_info()[0], sys.exc_info()[1], None)
         sys.exit("Error: Mozilla directory renaming failed!")
 
+    # Does the user have a pre-existing backup repository?
+    old_backup_repository = os.path.join(topsrcdir, SWITCH_MOZILLA_REPO_OLD_REPO_LOCATION)
+    if SWITCH_MOZILLA_REPO_OLD_REPO_LOCATION != "" and \
+            os.path.exists(old_backup_repository):
+        # Yes, so let's use that
+        print "Moving " + old_backup_repository + " to " + mozilla_path + "..."
+        try:
+            os.rename(old_backup_repository, mozilla_path)
+        except:
+            # Print the exception without its traceback.
+            sys.excepthook(sys.exc_info()[0], sys.exc_info()[1], None)
+            sys.exit("Error: Renaming old backup directory failed! You must recover manually.")
+        # Let's leave the hgrc as it was, so any repo specific config is left
+        # the same.
+        return
+
     # Locally clone common repository history.
-    check_call_noisy([options.hg, 'clone', '-r', MOZILLA_BASE_REV] + hgopts + [mozilla_trunk_path, mozilla_path])
+    check_call_noisy([options.hg, 'clone', '-r', SWITCH_MOZILLA_BASE_REV] + hgcloneopts + hgopts + [backup_mozilla_path, mozilla_path],
+                     retryMax=options.retries)
 
     # Rewrite hgrc for new local mozilla repo based on pre-existing hgrc
     # but with new values
@@ -166,25 +261,37 @@ def backup_cvs_extension(extensionName, extensionDir, extensionPath):
         sys.exit("Error: %s directory renaming failed!" % extensionName)
 
 def do_hg_pull(dir, repository, hg, rev):
+    """Clone if the dir doesn't exist, pull if it does.
+    """
+
     fulldir = os.path.join(topsrcdir, dir)
-    # clone if the dir doesn't exist, pull if it does
+
+    hgcloneopts = []
+    if options.hgcloneopts:
+        hgcloneopts = options.hgcloneopts.split()
+
     hgopts = []
     if options.hgopts:
         hgopts = options.hgopts.split()
+
     if not os.path.exists(fulldir):
         fulldir = os.path.join(topsrcdir, dir)
-        check_call_noisy([hg, 'clone'] + hgopts + [repository, fulldir])
+        check_call_noisy([hg, 'clone'] + hgcloneopts + hgopts + [repository, fulldir],
+                         retryMax=options.retries)
     else:
         cmd = [hg, 'pull', '-R', fulldir] + hgopts
         if repository is not None:
             cmd.append(repository)
-        check_call_noisy(cmd)
+        check_call_noisy(cmd, retryMax=options.retries)
+
     # update to specific revision
+    cmd = [hg, 'update', '-r', rev, '-R', fulldir ] + hgopts
     if options.verbose:
-        cmd = [hg, 'update', '-v', '-r', rev, '-R', fulldir ] + hgopts
-    else:
-        cmd = [hg, 'update', '-r', rev, '-R', fulldir ] + hgopts
-    check_call_noisy(cmd)
+        cmd.append('-v')
+    # Explicitly never retry 'hg update': otherwise any merge failures are ignored.
+    # This command is local: a failure can't be caused by a network error.
+    check_call_noisy(cmd, retryMax=0)
+
     check_call([hg, 'parent', '-R', fulldir,
                 '--template=Updated to revision {node}.\n'])
 
@@ -195,19 +302,22 @@ def do_cvs_checkout(modules, tag, cvsroot, cvs, checkoutdir):
     for module in modules:
         (parent, leaf) = os.path.split(module)
         print "CVS checkout begin: " + datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S UTC")
-        if tag == 'HEAD':
-            check_call_noisy([cvs, '-d', cvsroot, '-q',
-                              'checkout', '-P', '-A', '-d', leaf,
-                              'mozilla/%s' % module],
-                             cwd=os.path.join(topsrcdir, checkoutdir, parent))
-        else:
-            check_call_noisy([cvs, '-d', cvsroot, '-q',
-                              'checkout', '-P', '-r', tag, '-d', leaf,
-                              'mozilla/%s' % module],
-                             cwd=os.path.join(topsrcdir, checkoutdir, parent))
+        revopt = ((tag == 'HEAD') and ['-A']) or ['-r', tag]
+        # Explicitly never retry 'cvs checkout': otherwise any merge failures are ignored.
+        # No way to tell whether a failure is caused by a network error or a local conflict: better be safe.
+        check_call_noisy([cvs, '-d', cvsroot, '-q',
+                          'checkout', '-P'] + revopt + ['-d', leaf,
+                          'mozilla/%s' % module],
+                         retryMax=0,
+                         cwd=os.path.join(topsrcdir, checkoutdir, parent))
         print "CVS checkout end: " + datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S UTC")
 
-o = OptionParser(usage="client.py [options] checkout")
+def check_retries_option(option, opt_str, value, parser):
+  if value < 0:
+    raise OptionValueError("%s option value needs to be positive (not '%d')" % (opt_str, value))
+  setattr(parser.values, option.dest, value)
+
+o = OptionParser(usage="%prog [options] checkout")
 o.add_option("-m", "--comm-repo", dest="comm_repo",
              default=None,
              help="URL of comm (Calendar/Mail/Suite) repository to pull from (default: use hg default in .hg/hgrc)")
@@ -252,7 +362,6 @@ o.add_option("--chatzilla-rev", dest = "chatzilla_rev",
              default = DEFAULT_CHATZILLA_REV,
              help = "Revision of ChatZilla repository to update to. Default: \"" + DEFAULT_CHATZILLA_REV + "\"")
 
-
 o.add_option("--venkman-repo", dest = "venkman_repo",
              default = None,
              help = "URL of Venkman repository to pull from (default: use hg default in mozilla/extensions/venkman/.hg/hgrc; or if that file doesn't exist, use \"" + DEFAULT_VENKMAN_REPO + "\".)")
@@ -265,16 +374,23 @@ o.add_option("--venkman-rev", dest = "venkman_rev",
 
 o.add_option("--hg", dest="hg", default=os.environ.get('HG', 'hg'),
              help="The location of the hg binary")
-o.add_option("--cvs", dest="cvs", default=os.environ.get('CVS', 'cvs'),
-             help="The location of the cvs binary")
-o.add_option("--cvsroot", dest="cvsroot",
-             default=os.environ.get('CVSROOT', ':pserver:anonymous@cvs-mirror.mozilla.org:/cvsroot'),
-             help="The CVSROOT (default: :pserver:anonymous@cvs-mirror.mozilla.org:/cvsroot")
 o.add_option("-v", "--verbose", dest="verbose",
              action="store_true", default=False,
              help="Enable verbose output on hg updates")
 o.add_option("--hg-options", dest="hgopts",
              help="Pass arbitrary options to hg commands (i.e. --debug, --time)")
+o.add_option("--hg-clone-options", dest="hgcloneopts",
+             help="Pass arbitrary options to hg clone commands (i.e. --uncompressed)")
+
+o.add_option("--cvs", dest="cvs", default=os.environ.get('CVS', 'cvs'),
+             help="The location of the cvs binary")
+o.add_option("--cvsroot", dest="cvsroot",
+             default=os.environ.get('CVSROOT', ':pserver:anonymous@cvs-mirror.mozilla.org:/cvsroot'),
+             help="The CVSROOT (default: :pserver:anonymous@cvs-mirror.mozilla.org:/cvsroot")
+
+o.add_option("--retries", dest="retries", type="int", metavar="NUM",
+             default=1, help="Number of times to retry a failed command before giving up. (default: 1)",
+             action="callback", callback=check_retries_option)
 
 def fixup_comm_repo_options(options):
     """Check options.comm_repo value.
